@@ -8,10 +8,11 @@
 from collections import defaultdict
 import concurrent.futures
 
-from PIL import Image
 import numpy as np
+from PIL import Image
 
 from pyxllib.basic import *
+from pyxllib.cv import cv2pil, PilPrcs
 
 
 def get_img_content(in_):
@@ -32,12 +33,11 @@ def get_img_content(in_):
     elif isinstance(in_, Image.Image):
         img = in_
     elif isinstance(in_, np.ndarray):
-        from pyxllib.cv import cv2pil
         img = cv2pil(in_)
     else:
         raise ValueError(f'{type(in_)}')
 
-    img = image_rgba2rgb(img)  # 如果是RGBA类型，要把透明底变成白色
+    img = PilPrcs.rgba2rgb(img)  # 如果是RGBA类型，要把透明底变成白色
     file = io.BytesIO()
     img.save(file, 'JPEG')
     content = file.getvalue()
@@ -68,7 +68,7 @@ def magick(infile, *, outfile=None, if_exists='error', transparent=None, trim=Fa
 
     # 2
     # 200914周一20:40，这有个相对路径的bug，修复了下，否则 test/a.png 会变成 test/test/a.png
-    if File(outfile).exist_preprcs(if_exists, exclude=File(infile)):
+    if File(outfile).exist_preprcs(if_exists):
         # 2.1 判断是否是支持的输入文件类型
         ext = os.path.splitext(infile)[1].lower()
         if not File(infile) or not ext in ('.png', '.eps', '.pdf', '.jpg', '.jpeg', '.wmf', '.emf'):
@@ -161,7 +161,8 @@ def ensure_pngs(folder, *, if_exists='skip',
 
 
 def zoomsvg(file, scale=1):
-    """
+    """ 缩放svg文件
+
     :param file:
         如果输入一个目录，会处理目录下所有的svg图片
         否则只处理指定的文件
@@ -187,43 +188,3 @@ def zoomsvg(file, scale=1):
             File(file).write(s, if_exists='delete')
     elif isinstance(file, str) and '<svg ' in file:  # 输入svg的代码文本
         return re.sub(r'<svg .+?>', func, file, flags=re.DOTALL)
-
-
-def reduce_image_filesize(path, filesize):
-    """
-    :param path: 图片路径，支持png、jpg等多种格式
-    :param filesize: 单位Bytes
-        可以用 300*1024 来表示 300KB
-    :return:
-
-    >> reduce_image_filesize('a.jpg', 300*1024)
-    """
-    from PIL import Image
-
-    path = File(path)
-    # 1 无论什么情况，都先做个100%的resize处理，很可能会去掉一些没用的冗余信息
-    im = Image.open(f'{path}')
-    im.resize(im.size).save(f'{path}')
-
-    # 2 然后开始循环处理
-    while True:
-        r = path.size / filesize
-        if r <= 1: break
-        # 假设图片面积和文件大小成正比，如果r=4，表示长宽要各减小至1/(r**0.5)才能到目标文件大小
-        rate = min(1 / (r ** 0.5), 0.95)  # 并且限制每轮至少要缩小至95%，避免可能会迭代太多轮
-        im = Image.open(f'{path}')
-        im.resize((int(im.size[0] * rate), int(im.size[1] * rate))).save(f'{path}')
-
-
-def image_rgba2rgb(im):
-    if im.mode in ('RGBA', 'P'):
-        # 判断图片mode模式，如果是RGBA或P等可能有透明底，则和一个白底图片合成去除透明底
-        background = Image.new('RGBA', im.size, (255, 255, 255))
-        # composite是合成的意思。将右图的alpha替换为左图内容
-        im = Image.alpha_composite(background, im.convert('RGBA')).convert('RGB')
-    return im
-
-
-____section_temp = """
-临时添加的新功能
-"""
